@@ -138,8 +138,8 @@ static void printcopyright(FILE *fp) {
 		   "                   Robert M. Funk, Graham Wilson\n"
 		   "Copyright (C) 2005 - 2012 Sunil Shetye\n"
 		   "Copyright (C) 2005 - %d Matthias Andree\n"
-		   ), 2024);
-	fprintf(fp, GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
+		   ), 2025);
+	fprintf(fp, "%s", GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
 		   "are welcome to redistribute it under certain conditions. For details,\n"
 		   "please see the file COPYING in the source or documentation directory.\n"));
 #if defined(SSL_ENABLE) && !defined(USING_WOLFSSL)
@@ -343,9 +343,13 @@ int main(int argc, char **argv)
 		   "Run-time uses SSL library %#lx \"%s\"\n"),
 			OPENSSL_VERSION_NUMBER, OPENSSL_VERSION_TEXT,
 			OpenSSL_version_num(), OpenSSL_version(OPENSSL_VERSION));
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L // OpenSSL_version API in this form added in OpenSSL 3.0 according to manpage
+# if defined(OPENSSL_DIR) && defined(OPENSSL_ENGINES_DIR) // in-depth fix in case some OpenSSL freerider claims 3.x compat without delivering it
 	printf(GT_("OpenSSL: %s\nEngines: %s\n"),
 			OpenSSL_version(OPENSSL_DIR),
 			OpenSSL_version(OPENSSL_ENGINES_DIR));
+# endif
+#endif
 # if !HAVE_DECL_TLS1_3_VERSION || defined(OPENSSL_NO_TLS1_3)
 # error Your SSL/TLS library does not support TLS v1.3.
 # endif
