@@ -10,9 +10,7 @@
 #include  <stdio.h>
 #include  <string.h>
 #include  <ctype.h>
-#if defined(STDC_HEADERS)
 #include  <stdlib.h>
-#endif
 #include  "socket.h"
 
 #include  "i18n.h"
@@ -81,6 +79,8 @@ int do_otp(int sock, const char *command, struct query *ctl)
 
     return PS_SUCCESS;
 }
+#else
+typedef int make_pedantic_iso_c_compilers_happy;
 #endif /* OPIE_ENABLE */
 
 /* opie.c ends here */
