@@ -6,22 +6,21 @@
  */
 
 #include "config.h"
+#include "fetchmail.h"
 
 #include <stdio.h>
 #include <pwd.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
-#if defined(STDC_HEADERS)
-#include  <stdlib.h>
-#include  <limits.h>
-#else
-#include  <ctype.h>
-#endif
+#include <stdlib.h>
+#include <limits.h>
 
 #include "getopt.h"
-#include "fetchmail.h"
 #include "i18n.h"
 
+#include "tunable.h"
+
 enum {
     LA_INVISIBLE = 256,
     LA_PIDFILE,
@@ -41,10 +40,12 @@ enum {
     LA_PRINCIPAL,
     LA_TRACEPOLLS,
     LA_SSL,
+    LA_NOSSL,
     LA_SSLKEY,
     LA_SSLCERT,
     LA_SSLPROTO,
     LA_SSLCERTCK,
+    LA_NOSSLCERTCK,
     LA_SSLCERTFILE,
     LA_SSLCERTPATH,
     LA_SSLCOMMONNAME,
@@ -53,14 +54,17 @@ enum {
     LA_FASTUIDL,
     LA_LIMITFLUSH,
     LA_IDLE,
+    LA_FORCEIDLE,
     LA_NOSOFTBOUNCE,
     LA_SOFTBOUNCE,
-    LA_BADHEADER
+    LA_BADHEADER,
+    LA_IDLETIMEOUT,
+    LA_MOVETO
 };
 
-/* options still left: CgGhHjJoORTWxXYz */
+/* options still left: ACgGhHjJoORTWxXYz */
 static const char *shortoptions = 
-	"?Vcsvd:NqL:f:i:p:UP:A:t:E:Q:u:akKFnl:r:S:Z:b:B:e:m:I:M:yw:D:";
+	"?Vcsvd:NqL:f:i:p:UP:t:E:Q:u:akKFnl:r:S:Z:b:B:e:m:I:M:yw:D:";
 
 static const struct option longoptions[] = {
 /* this can be const because all flag fields are 0 and will never get set */
@@ -89,6 +93,8 @@ static const struct option longoptions[] = {
   {"proto",	required_argument, (int *) 0, 'p' },
   {"uidl",	no_argument,	   (int *) 0, 'U' },
   {"idle",	no_argument,	   (int *) 0, LA_IDLE},
+  {"forceidle",	no_argument,	   (int *) 0, LA_FORCEIDLE},
+  {"idletimeout",required_argument,(int *) 0, LA_IDLETIMEOUT },
   {"port",	required_argument, (int *) 0, 'P' },
   {"service",	required_argument, (int *) 0, 'P' },
   {"auth",	required_argument, (int *) 0, LA_AUTH},
@@ -104,6 +110,7 @@ static const struct option longoptions[] = {
   {"fetchall",	no_argument,	   (int *) 0, 'a' },
   {"nokeep",	no_argument,	   (int *) 0, 'K' },
   {"keep",	no_argument,	   (int *) 0, 'k' },
+  {"moveto",    required_argument, (int *) 0, LA_MOVETO },
   {"flush",	no_argument,	   (int *) 0, 'F' },
   {"limitflush",	no_argument, (int *) 0, LA_LIMITFLUSH },
   {"norewrite",	no_argument,	   (int *) 0, 'n' },
@@ -128,10 +135,12 @@ static const struct option longoptions[] = {
 
 #ifdef SSL_ENABLE
   {"ssl",	no_argument,	   (int *) 0, LA_SSL },
+  {"nossl",	no_argument,	   (int *) 0, LA_NOSSL },
   {"sslkey",	required_argument, (int *) 0, LA_SSLKEY },
   {"sslcert",	required_argument, (int *) 0, LA_SSLCERT },
   {"sslproto",	 required_argument, (int *) 0, LA_SSLPROTO },
   {"sslcertck", no_argument,	   (int *) 0, LA_SSLCERTCK },
+  {"nosslcertck", no_argument,	   (int *) 0, LA_NOSSLCERTCK },
   {"sslcertfile",   required_argument, (int *) 0, LA_SSLCERTFILE },
   {"sslcertpath",   required_argument, (int *) 0, LA_SSLCERTPATH },
   {"sslcommonname",    required_argument, (int *) 0, LA_SSLCOMMONNAME },
@@ -159,7 +168,6 @@ static const struct option longoptions[] = {
 static int xatoi(char *s, int *errflagptr)
 /* do safe conversion from string to number */
 {
-#if defined (STDC_HEADERS) && defined (LONG_MAX) && defined (INT_MAX)
     /* parse and convert numbers, but also check for invalid characters in
      * numbers
      */
@@ -190,48 +198,16 @@ static int xatoi(char *s, int *errflagptr)
     }
 
     return (int) value;  /* shut up, I know what I'm doing */
-#else
-    int	i;
-    char *dp;
-# if defined (STDC_HEADERS)
-    size_t	len;
-# else
-    int		len;
-# endif
-
-    /* We do only base 10 conversions here (atoi)! */
-
-    len = strlen(s);
-    /* check for leading white spaces */
-    for (i = 0; (i < len) && isspace((unsigned char)s[i]); i++)
-    	;
-
-    dp = &s[i];
-
-    /* check for +/- */
-    if (i < len && (s[i] == '+' || s[i] == '-'))	i++;
-
-    /* skip over digits */
-    for ( /* no init */ ; (i < len) && isdigit((unsigned char)s[i]); i++)
-    	;
-
-    /* check for trailing garbage */
-    if (i != len) {
-    	(void) fprintf(stderr, GT_("String '%s' is not a valid number string.\n"), s);
-    	(*errflagptr)++;
-	return 0;
-    }
-
-    /* atoi should be safe by now, except for number range over/underflow */
-    return atoi(dp);
-#endif
 }
 
 /** parse and validate the command line options */
 int parsecmdline (int argc /** argument count */,
 		  char **argv /** argument strings */,
 		  struct runctl *rctl /** global run controls to modify */,
-		  struct query *ctl /** option record to initialize */)
+		  struct query *ctl /** option record to initialize */,
+		  flag *safewithbg /** set to whether options are
+				     compatible with another copy
+				     running in the background */)
 {
     /*
      * return value: if positive, argv index of last parsed option + 1
@@ -246,9 +222,12 @@ int parsecmdline (int argc /** argument count */,
     int errflag = 0;	/* TRUE when a syntax error is detected */
     int helpflag = 0;	/* TRUE when option help was explicitly requested */
     int option_index;
+    int option_safe;	/* to track if option currently parsed is safe
+			   with a background copy */
     char *buf, *cp;
 
     rctl->poll_interval = -1;
+    *safewithbg = TRUE;
 
     memset(ctl, '\0', sizeof(struct query));    /* start clean */
     ctl->smtp_socket = -1;
@@ -257,24 +236,30 @@ int parsecmdline (int argc /** argument count */,
 	   (c = getopt_long(argc,argv,shortoptions,
 			    longoptions, &option_index)) != -1)
     {
+	option_safe = FALSE;
+
 	switch (c) {
 	case 'V':
 	    versioninfo = TRUE;
+	    option_safe = TRUE;
 	    break;
 	case 'c':
 	    check_only = TRUE;
 	    break;
 	case 's':
 	    outlevel = O_SILENT;
+	    option_safe = 1;
 	    break;
 	case 'v':
 	    if (outlevel >= O_VERBOSE)
 		outlevel = O_DEBUG;
 	    else
 		outlevel = O_VERBOSE;
+	    option_safe = TRUE;
 	    break;
 	case 'd':
-	    rctl->poll_interval = xatoi(optarg, &errflag);
+	    c = xatoi(optarg, &errflag);
+	    rctl->poll_interval = NUM_VALUE_IN(c);
 	    break;
 	case 'N':
 	    nodetach = TRUE;
@@ -347,7 +332,7 @@ int parsecmdline (int argc /** argument count */,
 	    else if (strcasecmp(optarg,"kpop") == 0)
 	    {
 		ctl->server.protocol = P_POP3;
-		ctl->server.service = KPOP_PORT;
+		ctl->server.service = xstrdup(KPOP_PORT);
 #ifdef KERBEROS_V5
 		ctl->server.authenticate =  A_KERBEROS_V5;
 #else
@@ -371,6 +356,9 @@ int parsecmdline (int argc /** argument count */,
 	case LA_IDLE:
 	    ctl->idle = FLAG_TRUE;
 	    break;
+	case LA_FORCEIDLE:
+	    ctl->forceidle = FLAG_TRUE;
+	    break;
 	case 'P':
 	    ctl->server.service = optarg;
 	    break;
@@ -387,8 +375,8 @@ int parsecmdline (int argc /** argument count */,
 		ctl->server.authenticate = A_KERBEROS_V5;
 	    else if (strcmp(optarg, "kerberos_v4") == 0)
 		ctl->server.authenticate = A_KERBEROS_V4;
-	    else if (strcmp(optarg, "ssh") == 0)
-		ctl->server.authenticate = A_SSH;
+	    else if (0 == strcmp(optarg, "implicit") || 0 == strcmp(optarg, "ssh"))
+		ctl->server.authenticate = A_IMPLICIT;
 	    else if (strcasecmp(optarg, "external") == 0)
 		ctl->server.authenticate = A_EXTERNAL;
 	    else if (strcmp(optarg, "otp") == 0)
@@ -546,6 +534,10 @@ int parsecmdline (int argc /** argument count */,
 	    ctl->use_ssl = FLAG_TRUE;
 	    break;
 
+	case LA_NOSSL:
+	    ctl->use_ssl = FLAG_FALSE;
+	    break;
+
 	case LA_SSLKEY:
 	    ctl->sslkey = prependdir (optarg, currentwd);
 	    break;
@@ -562,6 +554,10 @@ int parsecmdline (int argc /** argument count */,
 	    ctl->sslcertck = FLAG_TRUE;
 	    break;
 
+	case LA_NOSSLCERTCK:
+	    ctl->sslcertck = FLAG_FALSE;
+	    break;
+
 	case LA_SSLCERTFILE:
 	    ctl->sslcertfile = prependdir(optarg, currentwd);
 	    break;
@@ -594,6 +590,7 @@ int parsecmdline (int argc /** argument count */,
 
 	case LA_CONFIGDUMP:
 	    configdump = TRUE;
+	    option_safe = TRUE;
 	    break;
 
 	case LA_SYSLOG:
@@ -608,15 +605,27 @@ int parsecmdline (int argc /** argument count */,
 	    ctl->server.tracepolls = FLAG_TRUE;
 	    break;
 
+	case LA_IDLETIMEOUT:
+	    c = xatoi(optarg, &errflag);
+	    ctl->server.idle_timeout = NUM_VALUE_IN(c);
+	    break;
+
+	case LA_MOVETO:
+	    ctl->moveto = xstrdup(optarg);
+	    break;
+
 	case '?':
+	    helpflag = 1;
 	default:
-	    helpflag++;
+	    break;
 	}
+	*safewithbg &= option_safe;
     }
 
     if (errflag || ocount > 1 || helpflag) {
 	/* squawk if syntax errors were detected */
 #define P(s)    fputs(s, helpflag ? stdout : stderr)
+#define PS()    (helpflag ? stdout : stderr)
 	P(GT_("usage:  fetchmail [options] [server ...]\n"));
 	P(GT_("  Options are as follows:\n"));
 	P(GT_("  -?, --help        display this option help\n"));
@@ -630,6 +639,7 @@ int parsecmdline (int argc /** argument count */,
 	P(GT_("  -q, --quit        kill daemon process\n"));
 	P(GT_("  -L, --logfile     specify logfile name\n"));
 	P(GT_("      --syslog      use syslog(3) for most messages when running as a daemon\n"));
+	P(GT_("      --nosyslog    turns off use of syslog(3)\n"));
 	P(GT_("      --invisible   don't write Received & enable host spoofing\n"));
 	P(GT_("  -f, --fetchmailrc specify alternate run control file\n"));
 	P(GT_("  -i, --idfile      specify alternate UIDs file\n"));
@@ -643,36 +653,42 @@ int parsecmdline (int argc /** argument count */,
 	P(GT_("  -M, --monitor     monitor interface for activity\n"));
 #endif
 #if defined( SSL_ENABLE )
-	P(GT_("      --ssl         enable ssl encrypted session\n"));
+	P(GT_("      --ssl         enable SSL/TLS-wrapped encrypted session\n"));
+	P(GT_("      --nossl       disable SSL/TLS-wrapped encrypted session\n"));
 	P(GT_("      --sslkey      ssl private key file\n"));
 	P(GT_("      --sslcert     ssl client certificate\n"));
 	P(GT_("      --sslcertck   do strict server certificate check (recommended)\n"));
+	P(GT_("      --nosslcertck skip strict server certificate check (insecure)\n"));
 	P(GT_("      --sslcertfile path to trusted-CA ssl certificate file\n"));
 	P(GT_("      --sslcertpath path to trusted-CA ssl certificate directory\n"));
 	P(GT_("      --sslcommonname  expect this CommonName from server (discouraged)\n"));
 	P(GT_("      --sslfingerprint fingerprint that must match that of the server's cert.\n"));
-	P(GT_("      --sslproto    force ssl protocol (SSL2/SSL3/TLS1)\n"));
+	P(GT_("      --sslproto    force ssl protocol (see manual) and/or STLS/STARTTLS\n"));
 #endif
 	P(GT_("      --plugin      specify external command to open connection\n"));
 	P(GT_("      --plugout     specify external command to open smtp connection\n"));
 	P(GT_("      --bad-header {reject|accept}\n"
 	      "                    specify policy for handling messages with bad headers\n"));
 
-	P(GT_("  -p, --protocol    specify retrieval protocol (see man page)\n"));
+	P(GT_("  -p, --proto[col]  specify retrieval protocol (see man page)\n"));
 	P(GT_("  -U, --uidl        force the use of UIDLs (pop3 only)\n"));
+	P(GT_("      --idle        tells the IMAP server to send notice of new messages\n"));
+	P(GT_("      --forceidle   Force idle mode, even if server does not advertise the capability\n"));
+	fprintf(PS(), GT_("      --idletimeout specify timeout before refreshing --idle (%d s)\n"), CLIENT_IDLE_TIMEOUT);
 	P(GT_("      --port        TCP port to connect to (obsolete, use --service)\n"));
 	P(GT_("  -P, --service     TCP service to connect to (can be numeric TCP port)\n"));
-	P(GT_("      --auth        authentication type (password/kerberos/ssh/otp)\n"));
+	P(GT_("      --auth        authentication type (see manual)\n"));
 	P(GT_("  -t, --timeout     server nonresponse timeout\n"));
 	P(GT_("  -E, --envelope    envelope address header\n"));
 	P(GT_("  -Q, --qvirtual    prefix to remove from local user id\n"));
 	P(GT_("      --principal   mail service principal\n"));
 	P(GT_("      --tracepolls  add poll-tracing information to Received header\n"));
 
-	P(GT_("  -u, --username    specify users's login on server\n"));
+	P(GT_("  -u, --user[name]  specify users's login on server\n"));
 	P(GT_("  -a, --[fetch]all  retrieve old and new messages\n"));
 	P(GT_("  -K, --nokeep      delete new messages after retrieval\n"));
 	P(GT_("  -k, --keep        save new messages after retrieval\n"));
+	P(GT_("      --moveto      move message to the given IMAP folder instead of deleting it\n"));
 	P(GT_("  -F, --flush       delete old messages from server\n"));
 	P(GT_("      --limitflush  delete oversized messages\n"));
 	P(GT_("  -n, --norewrite   don't rewrite header addresses\n"));
@@ -683,7 +699,7 @@ int parsecmdline (int argc /** argument count */,
 	P(GT_("      --fetchdomains fetch mail for specified domains\n"));
 	P(GT_("  -D, --smtpaddress set SMTP delivery domain to use\n"));
 	P(GT_("      --smtpname    set SMTP full name username@domain\n"));
-	P(GT_("  -Z, --antispam,   set antispam response values\n"));
+	P(GT_("  -Z, --antispam    set antispam response values\n"));
 	P(GT_("  -b, --batchlimit  set batch limit for SMTP connections\n"));
 	P(GT_("  -B, --fetchlimit  set fetch limit for server connections\n"));
 	P(GT_("      --fetchsizelimit set fetch message size limit\n"));
