@@ -8,18 +8,15 @@
 #include  "config.h"
 
 #ifdef POP3_ENABLE
+#include  "fetchmail.h"
 #include  <stdio.h>
 #include  <string.h>
+#include  <strings.h>
 #include  <ctype.h>
-#if defined(HAVE_UNISTD_H)
 #include <unistd.h>
-#endif
-#if defined(STDC_HEADERS)
 #include  <stdlib.h>
-#endif
 #include  <errno.h>
 
-#include  "fetchmail.h"
 #include  "socket.h"
 #include  "i18n.h"
 #include  "uid_db.h"
@@ -478,7 +475,7 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	} /* maybe_starttls() */
 #endif /* SSL_ENABLE */
 
-	if (ctl->server.authenticate == A_SSH) {
+	if (ctl->server.authenticate == A_IMPLICIT) {
 		return PS_SUCCESS;
 	}
 
@@ -978,7 +975,7 @@ static int pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 static int pop3_getrange(int sock, 
 			 struct query *ctl,
 			 const char *folder,
-			 int *countp, int *newp, int *bytes)
+			 int *countp, int *newp, unsigned long long *bytes)
 /* get range of messages to be fetched */
 {
     int ok;
@@ -1001,7 +998,7 @@ static int pop3_getrange(int sock,
     if (ok == 0) {
 	int asgn;
 
-	asgn = sscanf(buf,"%d %d", countp, bytes);
+	asgn = sscanf(buf,"%d %llu", countp, bytes);
 	if (asgn != 2)
 		return PS_PROTOCOL;
     } else
@@ -1019,7 +1016,7 @@ static int pop3_getrange(int sock,
     if (*countp > 0 && (!ctl->fetchall || ctl->server.uidl))
     {
 	int fastuidl;
-	char id [IDLEN+1];
+	char id [IDLEN+1] = "";
 
 	set_uid_db_num_pos_0(&ctl->oldsaved, *countp);
 	set_uid_db_num_pos_0(&ctl->newsaved, *countp);
@@ -1042,6 +1039,9 @@ static int pop3_getrange(int sock,
 	if (!ctl->server.uidl) {
 	    gen_send(sock, "LAST");
 	    ok = pop3_ok(sock, buf);
+	    if (ok && outlevel >= O_MONITOR) {
+		report(stderr, GT_("LAST is an obsolete command that many servers will not support. We will try modern means to find what messages are new.\n"));
+	    }
 	} else
 	    ok = 1;
 
@@ -1416,6 +1416,8 @@ int doPOP3 (struct query *ctl)
 
     return(do_protocol(ctl, &pop3));
 }
+#else
+typedef int make_pedantic_iso_c_compilers_happy;
 #endif /* POP3_ENABLE */
 
 /* pop3.c ends here */
