@@ -4,7 +4,10 @@
  * Concept due to Harry Hochheiser.  Implementation by ESR.  Cleanup and
  * strict RFC821 compliance by Cameron MacPherson.
  *
- * Copyright 1997 Eric S. Raymond, 2009 Matthias Andree
+ * Copyright 1997 Eric S. Raymond, 2009 - 2019 Matthias Andree
+ * Contribution 2004 by Phil Endecott (by way of Rob Funk)
+ * Contributions 2005, 2011 by Sunil Shetye
+ * Contributions 2012, 2021 by Earl Chew
  * For license terms, see the file COPYING in this directory.
  */
 
@@ -15,6 +18,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <strings.h>
 #include <signal.h>
 #include "socket.h"
 #include "smtp.h"
@@ -117,11 +121,14 @@ static void SMTP_auth(int sock, char smtp_mode, char *username, char *password,
 		snprintf(tmp, sizeof(tmp), "^%s^%s", username, password);
 
 		len = strlen(tmp);
-		for (c = len - 1; c >= 0; c--)
-		{
-			if (tmp[c] == '^')
-				tmp[c] = '\0';
-		}
+
+		/* Take care not to overflow the buffer */
+		c = 0;
+		tmp[c] = '\0';
+		c += 1 + strlen(username);
+		if (c < len)
+			tmp[c] = '\0';
+
 		to64frombits(b64buf, tmp, len, sizeof b64buf);
 		SockPrintf(sock, "AUTH PLAIN %s\r\n", b64buf);
 		SMTP_ok(sock, smtp_mode, TIMEOUT_DEFAULT);
