@@ -11,21 +11,16 @@ extern "C" {
 #endif
 
 /* xmalloc.c */
-#if defined(HAVE_VOIDPOINTER) || defined(__cplusplus)
-#define XMALLOCTYPE void
-#else
-#define XMALLOCTYPE char
-#endif
 
 #if !defined __GNUC__ || __GNUC__ < 2
 # define __attribute__(xyz)    /* Ignore. */
 #endif
 
 /** Allocate \a n characters of memory, abort program on failure. */
-XMALLOCTYPE *xmalloc(size_t n) __attribute__((malloc));
+void *xmalloc(size_t n) __attribute__((malloc));
 
 /** Reallocate \a n characters of memory, abort program on failure. */
-XMALLOCTYPE *xrealloc(/*@null@*/ void *, size_t n);
+void *xrealloc(/*@null@*/ void *, size_t n);
 
 /** Free memory at position \a p and set pointer \a p to NULL afterwards. */
 #define xfree(p) { if (p) { free(p); } (p) = 0; }
